@@ -7,33 +7,27 @@ import re
 import sys
 import weakref
 from collections import deque
+from collections.abc import Generator, Iterable, Sequence
 from contextlib import contextmanager, suppress
 from enum import Enum, IntEnum, auto
 from functools import lru_cache, partial
 from gettext import gettext as _
 from itertools import chain
+from re import Pattern
 from time import time_ns
 from typing import (
     TYPE_CHECKING,
     Any,
     Callable,
     Deque,
-    Dict,
-    Generator,
-    Iterable,
-    List,
     NamedTuple,
     Optional,
-    Pattern,
-    Sequence,
-    Tuple,
     Union,
 )
 
 from .child import ProcessDesc
 from .cli_stub import CLIOptions
 from .clipboard import ClipboardRequestManager, set_clipboard_string
-from .config import build_ansi_color_table
 from .constants import (
     appname,
     clear_handled_signals,
@@ -45,6 +39,7 @@ from .fast_data_types import (
     CURSOR_BEAM,
     CURSOR_BLOCK,
     CURSOR_UNDERLINE,
+    ESC_CSI,
     ESC_DCS,
     ESC_OSC,
     GLFW_MOD_CONTROL,
@@ -56,6 +51,7 @@ from .fast_data_types import (
     SCROLL_LINE,
     SCROLL_PAGE,
     Color,
+    ColorProfile,
     KeyEvent,
     Screen,
     add_timer,
@@ -68,6 +64,7 @@ from .fast_data_types import (
     encode_key_for_tty,
     get_boss,
     get_click_interval,
+    get_mouse_data_for_window,
     get_options,
     is_css_pointer_name_valid,
     last_focused_os_window_id,
@@ -88,28 +85,18 @@ from .fast_data_types import (
     wakeup_main_loop,
 )
 from .keys import keyboard_mode_name, mod_mask
-from .notify import (
-    NotificationCommand,
-    NotifyImplementation,
-    OnlyWhen,
-    Urgency,
-    handle_notification_cmd,
-    notify_with_command,
-    sanitize_identifier_pat,
-)
-from .options.types import Options
 from .rgb import to_color
 from .terminfo import get_capabilities
 from .types import MouseEvent, OverlayType, WindowGeometry, ac, run_once
 from .typing import BossType, ChildType, EdgeLiteral, TabType, TypedDict
 from .utils import (
+    color_as_int,
     docs_url,
     key_val_matcher,
     kitty_ansi_sanitizer_pat,
     log_error,
     open_cmd,
     open_url,
-    parse_color_set,
     path_from_osc7_url,
     resolve_custom_file,
     resolved_shell,
@@ -120,12 +107,13 @@ from .utils import (
     shlex_split,
 )
 
-MatchPatternType = Union[Pattern[str], Tuple[Pattern[str], Optional[Pattern[str]]]]
+MatchPatternType = Union[Pattern[str], tuple[Pattern[str], Optional[Pattern[str]]]]
 
 
 if TYPE_CHECKING:
     from kittens.tui.handler import OpenUrlHandler
 
+    from .fast_data_types import MousePosition
     from .file_transmission import FileTransmission
 
 
@@ -162,7 +150,7 @@ class CwdRequest:
             return window.get_cwd_of_root_child() or ''
         return window.get_cwd_of_child(oldest=self.request_type is CwdRequestType.oldest) or ''
 
-    def modify_argv_for_launch_with_cwd(self, argv: List[str], env: Optional[Dict[str, str]]=None) -> str:
+    def modify_argv_for_launch_with_cwd(self, argv: list[str], env: Optional[dict[str, str]]=None) -> str:
         window = self.window
         if not window:
             return ''
@@ -226,7 +214,7 @@ def decode_cmdline(x: str) -> str:
     ctype, sep, val = x.partition('=')
     if ctype == 'cmdline':
         return next(shlex_split(val, True))
-    if ctype == 'cmdline_url':
+    elif ctype == 'cmdline_url':
         from urllib.parse import unquote
         return unquote(val)
     return ''
@@ -239,15 +227,15 @@ class WindowDict(TypedDict):
     title: str
     pid: Optional[int]
     cwd: str
-    cmdline: List[str]
+    cmdline: list[str]
     last_reported_cmdline: str
     last_cmd_exit_status: int
-    env: Dict[str, str]
-    foreground_processes: List[ProcessDesc]
+    env: dict[str, str]
+    foreground_processes: list[ProcessDesc]
     is_self: bool
     lines: int
     columns: int
-    user_vars: Dict[str, str]
+    user_vars: dict[str, str]
     at_prompt: bool
     created_at: int
 
@@ -288,18 +276,18 @@ DYNAMIC_COLOR_CODES.update({k+100: v for k, v in DYNAMIC_COLOR_CODES.items()})
 
 class Watcher:
 
-    def __call__(self, boss: BossType, window: 'Window', data: Dict[str, Any]) -> None:
+    def __call__(self, boss: BossType, window: 'Window', data: dict[str, Any]) -> None:
         pass
 
 
 class Watchers:
 
-    on_resize: List[Watcher]
-    on_close: List[Watcher]
-    on_focus_change: List[Watcher]
-    on_set_user_var: List[Watcher]
-    on_title_change: List[Watcher]
-    on_cmd_startstop: List[Watcher]
+    on_resize: list[Watcher]
+    on_close: list[Watcher]
+    on_focus_change: list[Watcher]
+    on_set_user_var: list[Watcher]
+    on_title_change: list[Watcher]
+    on_cmd_startstop: list[Watcher]
 
     def __init__(self) -> None:
         self.on_resize = []
@@ -310,7 +298,7 @@ class Watchers:
         self.on_cmd_startstop = []
 
     def add(self, others: 'Watchers') -> None:
-        def merge(base: List[Watcher], other: List[Watcher]) -> None:
+        def merge(base: list[Watcher], other: list[Watcher]) -> None:
             for x in other:
                 if x not in base:
                     base.append(x)
@@ -341,12 +329,12 @@ class Watchers:
                     or self.on_set_user_var or self.on_title_change or self.on_cmd_startstop)
 
 
-def call_watchers(windowref: Callable[[], Optional['Window']], which: str, data: Dict[str, Any]) -> None:
+def call_watchers(windowref: Callable[[], Optional['Window']], which: str, data: dict[str, Any]) -> None:
 
     def callback(timer_id: Optional[int]) -> None:
         w = windowref()
         if w is not None:
-            watchers: List[Watcher] = getattr(w.watchers, which)
+            watchers: list[Watcher] = getattr(w.watchers, which)
             w.call_watchers(watchers, data)
 
     add_timer(callback, 0, False)
@@ -368,7 +356,7 @@ def as_text(
     alternate_screen: bool = False,
     add_cursor: bool = False
 ) -> str:
-    lines: List[str] = []
+    lines: list[str] = []
     add_history = add_history and not (screen.is_using_alternate_linebuf() ^ alternate_screen)
     if alternate_screen:
         f = screen.as_text_alternate
@@ -390,7 +378,7 @@ def as_text(
 
     if add_history:
         pht = pagerhist(screen, as_ansi, add_wrap_markers)
-        h: List[str] = [pht] if pht else []
+        h: list[str] = [pht] if pht else []
         screen.as_text_for_history_buf(h.append, as_ansi, add_wrap_markers)
         if h:
             if as_ansi:
@@ -406,19 +394,6 @@ def as_text(
 
 
 
-def setup_colors(screen: Screen, opts: Options) -> None:
-    screen.color_profile.update_ansi_color_table(build_ansi_color_table(opts))
-
-    def s(c: Optional[Color]) -> int:
-        return 0 if c is None else (0xff000000 | int(c))
-    screen.color_profile.set_configured_colors(
-        s(opts.foreground), s(opts.background),
-        s(opts.cursor), s(opts.cursor_text_color),
-        s(opts.selection_foreground), s(opts.selection_background),
-        s(opts.visual_bell_color)
-    )
-
-
 @run_once
 def load_paste_filter() -> Callable[[str], str]:
     import runpy
@@ -455,7 +430,7 @@ def text_sanitizer(as_ansi: bool, add_wrap_markers: bool) -> Callable[[str], str
 
 
 def cmd_output(screen: Screen, which: CommandOutput = CommandOutput.last_run, as_ansi: bool = False, add_wrap_markers: bool = False) -> str:
-    lines: List[str] = []
+    lines: list[str] = []
     search_in_pager_hist = screen.cmd_output(which, lines.append, as_ansi, add_wrap_markers)
     if search_in_pager_hist:
         pht = pagerhist(screen, as_ansi, add_wrap_markers, True)
@@ -472,13 +447,100 @@ def process_remote_print(msg: memoryview) -> str:
     return replace_c0_codes_except_nl_space_tab(base64_decode(msg)).decode('utf-8', 'replace')
 
 
+def transparent_background_color_control(cp: ColorProfile, responses: dict[str, str], index: int, key: str, sep: str, val: str) -> None:
+    if sep == '=':
+        if val == '?':
+            if index > 8:
+                responses[key] = '?'
+            else:
+                c = cp.get_transparent_background_color(index - 1)
+                if c is None:
+                    responses[key] = ''
+                else:
+                    opacity = max(0, min(c.alpha / 255.0, 1))
+                    responses[key] = f'rgb:{c.red:02x}/{c.green:02x}/{c.blue:02x}@{opacity:.4f}'
+        elif index <= 8:
+            col, _, o = val.partition('@')
+            try:
+                opacity = float(o)
+            except Exception:
+                opacity = -1.0
+            c = to_color(col)
+            if c is not None:
+                cp.set_transparent_background_color(index - 1, c, opacity)
+    elif index <= 8:
+        cp.set_transparent_background_color(index - 1)
+
+
+def color_control(cp: ColorProfile, code: int, value: Union[str, bytes, memoryview] = '') -> str:
+    if isinstance(value, (bytes, memoryview)):
+        value = str(value, 'utf-8', 'replace')
+    responses: dict[str, str] = {}
+    for rec in value.split(';'):
+        key, sep, val = rec.partition('=')
+        if key.startswith('transparent_background_color'):
+            index = int(key[len('transparent_background_color'):])
+            transparent_background_color_control(cp, responses, index, key, sep, val)
+            continue
+        attr = {
+            'foreground': 'default_fg', 'background': 'default_bg',
+            'selection_background': 'highlight_bg', 'selection_foreground': 'highlight_fg',
+            'cursor': 'cursor_color', 'cursor_text': 'cursor_text_color',
+            'visual_bell': 'visual_bell_color',
+        }.get(key, '')
+        colnum = -1
+        with suppress(Exception):
+            colnum = int(key)
+
+        def serialize_color(c: Optional[Color]) -> str:
+            return '' if c is None else f'rgb:{c.red:02x}/{c.green:02x}/{c.blue:02x}'
+
+        if sep == '=':
+            if val == '?':
+                if attr:
+                    c = getattr(cp, attr)
+                    responses[key] = serialize_color(c)
+                else:
+                    if 0 <= colnum <= 255:
+                        c = cp.as_color((colnum << 8) | 1)
+                        responses[key] = serialize_color(c)
+                    else:
+                        responses[key] = '?'
+            else:
+                if attr:
+                    if val:
+                        val = val.partition('@')[0]
+                        col = to_color(val)
+                        if col is not None:
+                            setattr(cp, attr, col)
+                    else:
+                        with suppress(TypeError):
+                            setattr(cp, attr, None)
+                else:
+                    if 0 <= colnum <= 255:
+                        val = val.partition('@')[0]
+                        col = to_color(val)
+                        if col is not None:
+                            cp.set_color(colnum, color_as_int(col))
+        else:
+            if attr:
+                delattr(cp, attr)
+            else:
+                if 0 <= colnum <= 255:
+                    cp.set_color(colnum, get_options().color_table[colnum])
+    if responses:
+        payload = ';'.join(f'{k}={v}' for k, v in responses.items())
+        return f'{code};{payload}'
+    return ''
+
+
 class EdgeWidths:
     left: Optional[float]
     top: Optional[float]
     right: Optional[float]
     bottom: Optional[float]
 
-    def __init__(self, serialized: Optional[Dict[str, Optional[float]]] = None):
+    def __init__(self, serialized: Optional[dict[str, Optional[float]]] = None):
         if serialized is not None:
             self.left = serialized['left']
             self.right = serialized['right']
@@ -487,7 +549,7 @@ class EdgeWidths:
         else:
             self.left = self.top = self.right = self.bottom = None
 
-    def serialize(self) -> Dict[str, Optional[float]]:
+    def serialize(self) -> dict[str, Optional[float]]:
         return {'left': self.left, 'right': self.right, 'top': self.top, 'bottom': self.bottom}
 
     def copy(self) -> 'EdgeWidths':
@@ -497,7 +559,7 @@ class EdgeWidths:
 class GlobalWatchers:
 
     def __init__(self) -> None:
-        self.options_spec: Optional[Dict[str, str]] = None
+        self.options_spec: Optional[dict[str, str]] = None
         self.ans = Watchers()
         self.extra = ''
 
@@ -550,7 +612,7 @@ class Window:
         copy_colors_from: Optional['Window'] = None,
         watchers: Optional[Watchers] = None,
         allow_remote_control: bool = False,
-        remote_control_passwords: Optional[Dict[str, Sequence[str]]] = None,
+        remote_control_passwords: Optional[dict[str, Sequence[str]]] = None,
     ):
         if watchers:
             self.watchers = watchers
@@ -562,19 +624,18 @@ class Window:
         self.last_resized_at = 0.
         self.started_at = monotonic()
         self.created_at = time_ns()
-        self.current_remote_data: List[str] = []
+        self.current_remote_data: list[str] = []
         self.current_mouse_event_button = 0
         self.current_clipboard_read_ask: Optional[bool] = None
-        self.prev_osc99_cmd = NotificationCommand()
         self.last_cmd_output_start_time = 0.
         self.open_url_handler: 'OpenUrlHandler' = None
         self.last_cmd_cmdline = ''
         self.last_cmd_exit_status = 0
-        self.actions_on_close: List[Callable[['Window'], None]] = []
-        self.actions_on_focus_change: List[Callable[['Window', bool], None]] = []
-        self.actions_on_removal: List[Callable[['Window'], None]] = []
-        self.current_marker_spec: Optional[Tuple[str, Union[str, Tuple[Tuple[int, str], ...]]]] = None
-        self.kitten_result_processors: List[Callable[['Window', Any], None]] = []
+        self.actions_on_close: list[Callable[['Window'], None]] = []
+        self.actions_on_focus_change: list[Callable[['Window', bool], None]] = []
+        self.actions_on_removal: list[Callable[['Window'], None]] = []
+        self.current_marker_spec: Optional[tuple[str, Union[str, tuple[tuple[int, str], ...]]]] = None
+        self.kitten_result_processors: list[Callable[['Window', Any], None]] = []
         self.child_is_launched = False
         self.last_reported_pty_size = (-1, -1, -1, -1)
         self.needs_attention = False
@@ -583,12 +644,12 @@ class Window:
         self.default_title = os.path.basename(child.argv[0] or appname)
         self.child_title = self.default_title
         self.title_stack: Deque[str] = deque(maxlen=10)
-        self.user_vars: Dict[str, str] = {}
+        self.user_vars: dict[str, str] = {}
         self.id: int = add_window(tab.os_window_id, tab.id, self.title)
         self.clipboard_request_manager = ClipboardRequestManager(self.id)
         self.margin = EdgeWidths()
         self.padding = EdgeWidths()
-        self.kitten_result: Optional[Dict[str, Any]] = None
+        self.kitten_result: Optional[dict[str, Any]] = None
         if not self.id:
             raise Exception(f'No tab with id: {tab.id} in OS Window: {tab.os_window_id} was found, or the window counter wrapped')
         self.tab_id = tab.id
@@ -604,12 +665,10 @@ class Window:
         self.screen: Screen = Screen(self, 24, 80, opts.scrollback_lines, cell_width, cell_height, self.id)
         if copy_colors_from is not None:
             self.screen.copy_colors_from(copy_colors_from.screen)
-        else:
-            setup_colors(self.screen, opts)
         self.remote_control_passwords = remote_control_passwords
         self.allow_remote_control = allow_remote_control
 
-    def remote_control_allowed(self, pcmd: Dict[str, Any], extra_data: Dict[str, Any]) -> bool:
+    def remote_control_allowed(self, pcmd: dict[str, Any], extra_data: dict[str, Any]) -> bool:
         if not self.allow_remote_control:
             return False
         from .remote_control import remote_control_allowed
@@ -680,9 +739,8 @@ class Window:
         return int(val)
 
     def apply_options(self, is_active: bool) -> None:
-        opts = get_options()
         self.update_effective_padding()
-        setup_colors(self.screen, opts)
+        self.screen.color_profile.reload_from_opts()
 
     @property
     def title(self) -> str:
@@ -712,7 +770,7 @@ class Window:
             'created_at': self.created_at,
         }
 
-    def serialize_state(self) -> Dict[str, Any]:
+    def serialize_state(self) -> dict[str, Any]:
         ans = {
             'version': 1,
             'id': self.id,
@@ -747,7 +805,7 @@ class Window:
         return tab.overlay_parent(self)
 
     @property
-    def current_colors(self) -> Dict[str, Optional[int]]:
+    def current_colors(self) -> dict[str, Union[int, None, tuple[tuple[Color, float], ...]]]:
         return self.screen.color_profile.as_dict()
 
     @property
@@ -862,6 +920,8 @@ class Window:
             boss = get_boss()
             boss.child_monitor.resize_pty(self.id, *current_pty_size)
             self.last_resized_at = monotonic()
+            self.last_reported_pty_size = current_pty_size
+            self.notify_child_of_resize()
             if not self.child_is_launched:
                 self.child.mark_terminal_ready()
                 self.child_is_launched = True
@@ -871,7 +931,6 @@ class Window:
                     print(f'[{now:.3f}] Child launched', file=sys.stderr)
             elif boss.args.debug_rendering:
                 print(f'[{monotonic():.3f}] SIGWINCH sent to child in window: {self.id} with size: {current_pty_size}', file=sys.stderr)
-            self.last_reported_pty_size = current_pty_size
         else:
             mark_os_window_dirty(self.os_window_id)
 
@@ -947,7 +1006,7 @@ class Window:
 
     @ac('debug', 'Show a dump of the current lines in the scrollback + screen with their line attributes')
     def dump_lines_with_attrs(self) -> None:
-        strings: List[str] = []
+        strings: list[str] = []
         self.screen.dump_lines_with_attrs(strings.append)
         text = ''.join(strings)
         get_boss().display_scrollback(self, text, title='Dump of lines', report_cursor=False)
@@ -1034,11 +1093,9 @@ class Window:
                 log_error(f'Ignoring unknown OSC 777: {raw_data}')
                 return  # unknown OSC 777
             raw_data = raw_data[len('notify;'):]
-        cmd = handle_notification_cmd(osc_code, raw_data, self.id, self.prev_osc99_cmd)
-        if cmd is not None and osc_code == 99:
-            self.prev_osc99_cmd = cmd
+        get_boss().notification_manager.handle_notification_cmd(self.id, osc_code, raw_data)
 
-    def on_mouse_event(self, event: Dict[str, Any]) -> bool:
+    def on_mouse_event(self, event: dict[str, Any]) -> bool:
         event['mods'] = event.get('mods', 0) & mod_mask
         ev = MouseEvent(**event)
         self.current_mouse_event_button = ev.button
@@ -1096,7 +1153,7 @@ class Window:
 
         from .utils import SSHConnectionData
         args = self.ssh_kitten_cmdline()
-        conn_data: Union[None, List[str], SSHConnectionData] = None
+        conn_data: Union[None, list[str], SSHConnectionData] = None
         if args:
             ssh_cmdline = sorted(self.child.foreground_processes, key=lambda p: p['pid'])[-1]['cmdline'] or ['']
             if 'ControlPath=' in ' '.join(ssh_cmdline):
@@ -1186,42 +1243,26 @@ class Window:
                     tab.relayout_borders()
                 tab.on_bell(self)
 
-    def change_colors(self, changes: Dict[DynamicColor, Optional[str]]) -> None:
-        dirtied = default_bg_changed = False
-
-        def item(raw: Optional[str]) -> int:
-            if raw is None:
-                return 0
-            v = to_color(raw)
-            if v is None:
-                return 0
-            return 0xff000000 | int(v)
-
-        for which, val_ in changes.items():
-            val = item(val_)
-            dirtied = True
-            setattr(self.screen.color_profile, which.name, val)
-            if which.name == 'default_bg':
-                default_bg_changed = True
-        if dirtied:
-            self.screen.mark_as_dirty()
-        if default_bg_changed:
-            get_boss().default_bg_changed_for(self.id)
-
     def color_profile_popped(self, bg_changed: bool) -> None:
         if bg_changed:
             get_boss().default_bg_changed_for(self.id)
 
-    def report_color(self, code: str, r: int, g: int, b: int) -> None:
+    def report_color(self, code: str, col: Color) -> None:
+        r, g, b = col.red, col.green, col.blue
         r |= r << 8
         g |= g << 8
         b |= b << 8
         self.screen.send_escape_code_to_child(ESC_OSC, f'{code};rgb:{r:04x}/{g:04x}/{b:04x}')
 
-    def report_notification_activated(self, identifier: str) -> None:
-        identifier = sanitize_identifier_pat().sub('', identifier)
-        self.screen.send_escape_code_to_child(ESC_OSC, f'99;i={identifier};')
+    def notify_child_of_resize(self) -> None:
+        pty_size = self.last_reported_pty_size
+        if pty_size[0] > -1 and self.screen.in_band_resize_notification:
+            self.screen.send_escape_code_to_child(ESC_CSI, f'48;{pty_size[0]};{pty_size[1]};{pty_size[3]};{pty_size[2]}t')
 
+    def color_control(self, code: int, value: Union[str, bytes, memoryview] = '') -> None:
+        response = color_control(self.screen.color_profile, code, value)
+        if response:
+            self.screen.send_escape_code_to_child(ESC_OSC, response)
 
     def set_dynamic_color(self, code: int, value: Union[str, bytes, memoryview] = '') -> None:
         if isinstance(value, (bytes, memoryview)):
@@ -1230,30 +1271,64 @@ class Window:
             ret = set_pointer_shape(self.screen, value, self.os_window_id)
             if ret:
                 self.screen.send_escape_code_to_child(ESC_OSC, '22:' + ret)
-        color_changes: Dict[DynamicColor, Optional[str]] = {}
+
+        dirtied = default_bg_changed = False
+        def change(which: DynamicColor, val: str) -> None:
+            nonlocal dirtied, default_bg_changed
+            dirtied = True
+            if which.name == 'default_bg':
+                default_bg_changed = True
+            v = to_color(val) if val else None
+            if v is None:
+                delattr(self.screen.color_profile, which.name)
+            else:
+                setattr(self.screen.color_profile, which.name, v)
+
         for val in value.split(';'):
             w = DYNAMIC_COLOR_CODES.get(code)
             if w is not None:
                 if val == '?':
-                    col = getattr(self.screen.color_profile, w.name)
-                    self.report_color(str(code), col >> 16, (col >> 8) & 0xff, col & 0xff)
+                    col = getattr(self.screen.color_profile, w.name) or Color()
+                    self.report_color(str(code), col)
                 else:
-                    q = None if code >= 100 else val
-                    color_changes[w] = q
+                    q = '' if code >= 100 else val
+                    change(w, q)
             code += 1
-        if color_changes:
-            self.change_colors(color_changes)
+        if dirtied:
+            self.screen.mark_as_dirty()
+        if default_bg_changed:
+            get_boss().default_bg_changed_for(self.id)
 
     def set_color_table_color(self, code: int, bvalue: Optional[memoryview] = None) -> None:
         value = str(bvalue or b'', 'utf-8', 'replace')
         cp = self.screen.color_profile
+
+        def parse_color_set(raw: str) -> Generator[tuple[int, Optional[int]], None, None]:
+            parts = raw.split(';')
+            lp = len(parts)
+            if lp % 2 != 0:
+                return
+            for c_, spec in [parts[i:i + 2] for i in range(0, len(parts), 2)]:
+                try:
+                    c = int(c_)
+                    if c < 0 or c > 255:
+                        continue
+                    if spec == '?':
+                        yield c, None
+                    else:
+                        q = to_color(spec)
+                        if q is not None:
+                            yield c, color_as_int(q)
+                except Exception:
+                    continue
+
         if code == 4:
             changed = False
             for c, val in parse_color_set(value):
                 if val is None:  # color query
                     qc = self.screen.color_profile.as_color((c << 8) | 1)
                     assert qc is not None
-                    self.report_color(f'4;{c}', qc.red, qc.green, qc.blue)
+                    self.report_color(f'4;{c}', qc)
                 else:
                     changed = True
                     cp.set_color(c, val)
@@ -1273,7 +1348,7 @@ class Window:
             self.refresh()
 
     def request_capabilities(self, q: str) -> None:
-        for result in get_capabilities(q, get_options()):
+        for result in get_capabilities(q, get_options(), self.id, self.os_window_id):
             self.screen.send_escape_code_to_child(ESC_DCS, result)
 
     def handle_remote_cmd(self, cmd: memoryview) -> None:
@@ -1408,13 +1483,13 @@ class Window:
     def handle_cmd_end(self, exit_status: str = '') -> None:
         if self.last_cmd_output_start_time == 0.:
             return
-        self.last_cmd_output_start_time = 0.
         try:
             self.last_cmd_exit_status = int(exit_status)
         except Exception:
             self.last_cmd_exit_status = 0
         end_time = monotonic()
         last_cmd_output_duration = end_time - self.last_cmd_output_start_time
+        self.last_cmd_output_start_time = 0.
 
         self.call_watchers(self.watchers.on_cmd_startstop, {
             "is_start": False, "time": end_time, 'cmdline': self.last_cmd_cmdline, 'exit_status': self.last_cmd_exit_status})
@@ -1423,30 +1498,21 @@ class Window:
         when, duration, action, notify_cmdline = opts.notify_on_cmd_finish
 
         if last_cmd_output_duration >= duration and when != 'never':
-            cmd = NotificationCommand()
+            from .notifications import OnlyWhen
+            nm = get_boss().notification_manager
+            cmd = nm.create_notification_cmd()
             cmd.title = 'kitty'
             s = self.last_cmd_cmdline.replace('\\\n', ' ')
             cmd.body = f'Command {s} finished with status: {exit_status}.\nClick to focus.'
-            cmd.actions = 'focus'
             cmd.only_when = OnlyWhen(when)
+            if not nm.is_notification_allowed(cmd, self.id):
+                return
             if action == 'notify':
-                notify_with_command(cmd, self.id)
+                nm.notify_with_command(cmd, self.id)
             elif action == 'bell':
-                class Bell(NotifyImplementation):
-                    def __init__(self, window_id: int):
-                        self.window_id = window_id
-                    def __call__(self, title: str, body: str, identifier: str, urgency: Urgency = Urgency.Normal) -> None:
-                        w = get_boss().window_id_map.get(self.window_id)
-                        if w:
-                            w.screen.bell()
-                notify_with_command(cmd, self.id, notify_implementation=Bell(self.id))
+                self.screen.bell()
             elif action == 'command':
-                class Run(NotifyImplementation):
-                    def __init__(self, last_cmd_cmdline: str):
-                        self.last_cmd_cmdline = last_cmd_cmdline
-                    def __call__(self, title: str, body: str, identifier: str, urgency: Urgency = Urgency.Normal) -> None:
-                        open_cmd([x.replace('%c', self.last_cmd_cmdline).replace('%s', exit_status) for x in notify_cmdline])
-                notify_with_command(cmd, self.id, notify_implementation=Run(self.last_cmd_cmdline))
+                open_cmd([x.replace('%c', self.last_cmd_cmdline).replace('%s', exit_status) for x in notify_cmdline])
             else:
                 raise ValueError(f'Unknown action in option `notify_on_cmd_finish`: {action}')
 
@@ -1548,7 +1614,7 @@ class Window:
     def has_selection(self) -> bool:
         return self.screen.has_selection()
 
-    def call_watchers(self, which: Iterable[Watcher], data: Dict[str, Any]) -> None:
+    def call_watchers(self, which: Iterable[Watcher], data: dict[str, Any]) -> None:
         boss = get_boss()
         for w in which:
             try:
@@ -1612,7 +1678,7 @@ class Window:
                 return True
         return False
 
-    def ssh_kitten_cmdline(self) -> List[str]:
+    def ssh_kitten_cmdline(self) -> list[str]:
         from kittens.ssh.utils import is_kitten_cmdline
         for p in self.child.foreground_processes:
             q = list(p['cmdline'] or ())
@@ -1729,6 +1795,10 @@ class Window:
         else:
             self.screen.erase_in_display(3 if scrollback else 2, False)
 
+    def current_mouse_position(self) -> Optional['MousePosition']:
+        ' Return the last position at which a mouse event was received by this window '
+        return get_mouse_data_for_window(self.os_window_id, self.tab_id, self.id)
+
     # actions {{{
 
     @ac('cp', 'Show scrollback in a pager like less')
@@ -1906,7 +1976,7 @@ class Window:
         return True
 
     @ac('mk', 'Toggle the current marker on/off')
-    def toggle_marker(self, ftype: str, spec: Union[str, Tuple[Tuple[int, str], ...]], flags: int) -> None:
+    def toggle_marker(self, ftype: str, spec: Union[str, tuple[tuple[int, str], ...]], flags: int) -> None:
         from .marks import marker_from_spec
         key = ftype, spec
         if key == self.current_marker_spec:
